<?php
error_reporting(E_ALL);
ini_set('display_errors', 1);

function fetchBinanceData($symbol, $interval, $limit) {
    $calculatedLimit = intval($limit);

    if ($calculatedLimit <= 0 || $calculatedLimit > 1000) {
        throw new Exception("مقدار limit نامعتبر است.");
    }

    $url = "https://api.binance.com/api/v3/klines?symbol=$symbol&interval=$interval&limit=$calculatedLimit";

    $response = @file_get_contents($url);
    if ($response === false) {
        throw new Exception("خطا در اتصال به API بایننس.");
    }

    $data = json_decode($response, true);
    if (!is_array($data) || count($data) < $limit) {
        throw new Exception("خطا در دریافت داده‌ها از API بایننس.");
    }

    $closePrices = [];
    $highPrices = [];
    $lowPrices = [];
    $timestamps = [];

    foreach ($data as $item) {
        $timestamps[] = $item[0] / 1000; // تبدیل به ثانیه
        $closePrices[] = floatval($item[4]);
        $highPrices[] = floatval($item[2]);
        $lowPrices[] = floatval($item[3]);
    }

    if (count($closePrices) < $limit) {
        throw new Exception("تعداد داده‌ها کافی نیست.");
    }

    return [$closePrices, $highPrices, $lowPrices, $timestamps];
}

function ema($data, $period) {
    if (count($data) < $period) {
        throw new Exception("EMA: داده‌ها کافی نیستند یا تعداد داده‌ها کمتر از دوره مشخص شده است.");
    }

    $k = 2 / ($period + 1);
    $ema = $data[0];
    for ($i = 1; $i < count($data); $i++) {
        $ema = ($data[$i] * $k) + ($ema * (1 - $k));
    }
    return $ema;
}

function sma($data, $period) {
    if (count($data) < $period) {
        throw new Exception("SMA: داده‌ها کافی نیستند یا تعداد داده‌ها کمتر از دوره مشخص شده است.");
    }

    return array_sum(array_slice($data, -$period)) / $period;
}

// --- الگوریتم‌های تریدرها ---
function traderAli($data, $high, $low) {
    $emaShort = ema($data, 20);
    $emaLong = ema($data, 50);
    $trend = ($emaShort > $emaLong) ? "صعودی" : "نزولی";
    $reason = "EMA کوتاه " . ($trend === "صعودی" ? "بالای" : "پایین") . " EMA بلند است.";
    return ["name" => "علی", "trend" => $trend, "reason" => $reason];
}

function traderReza($data, $high, $low) {
    $sma14 = sma($data, 14);
    $trend = (end($data) > $sma14) ? "صعودی" : "نزولی";
    $reason = "قیمت " . ($trend === "صعودی" ? "بالای" : "پایین") . " میانگین متحرک ساده 14 است.";
    return ["name" => "رضا", "trend" => $trend, "reason" => $reason];
}

function traderSara($data) {
    $rsi = sma($data, 14); // نمونه ساده RSI
    $trend = ($rsi > 50) ? "صعودی" : "نزولی";
    $reason = "RSI " . ($trend === "صعودی" ? "بالای" : "پایین") . " 50 است.";
    return ["name" => "سارا", "trend" => $trend, "reason" => $reason];
}

function traderNeda($data, $high, $low) {
    $pivotLow = min($low);
    $trend = (end($data) > $pivotLow) ? "صعودی" : "نزولی";
    $reason = "قیمت نزدیک به نقطه محوری پایین است.";
    return ["name" => "ندا", "trend" => $trend, "reason" => $reason];
}

function traderMohammad($data) {
    $roc = (end($data) - $data[count($data) - 10]) / $data[count($data) - 10];
    $trend = ($roc > 0) ? "صعودی" : "نزولی";
    $reason = "نرخ تغییر (ROC) " . ($trend === "صعودی" ? "مثبت" : "منفی") . " است.";
    return ["name" => "محمد", "trend" => $trend, "reason" => $reason];
}

function traderHamed($data, $high, $low) {
    $fibonacciRetracement = min($low) + 0.618 * (max($high) - min($low));
    $trend = (end($data) > $fibonacciRetracement) ? "صعودی" : "نزولی";
    $reason = "قیمت " . ($trend === "صعودی" ? "بالای" : "پایین") . " سطح بازگشت فیبوناچی است.";
    return ["name" => "حامد", "trend" => $trend, "reason" => $reason];
}

function traderFarhad($data) {
    $hma = sma($data, 9); // فرضی HMA
    $trend = (end($data) > $hma) ? "صعودی" : "نزولی";
    $reason = "میانگین متحرک هال " . ($trend === "صعودی" ? "بالا" : "پایین") . " است.";
    return ["name" => "فرهاد", "trend" => $trend, "reason" => $reason];
}

function traderArash($data, $high, $low) {
    $vwap = sma($data, 20);
    $trend = (end($data) > $vwap) ? "صعودی" : "نزولی";
    $reason = "قیمت " . ($trend === "صعودی" ? "بالای" : "پایین") . " VWAP است.";
    return ["name" => "آرش", "trend" => $trend, "reason" => $reason];
}

function traderElham($data) {
    $macdLine = ema($data, 12) - ema($data, 26);
    $signalLine = ema(array_slice($data, -9), 9);
    $trend = ($macdLine > $signalLine) ? "صعودی" : "نزولی";
    $reason = "خط MACD " . ($trend === "صعودی" ? "بالای" : "پایین") . " خط سیگنال است.";
    return ["name" => "الهام", "trend" => $trend, "reason" => $reason];
}

function traderShirin($data, $high, $low) {
    $atr = sma($high, 14) - sma($low, 14);
    $trend = (end($data) > $atr) ? "صعودی" : "نزولی";
    $reason = "قیمت " . ($trend === "صعودی" ? "بالای" : "پایین") . " ATR است.";
    return ["name" => "شیرین", "trend" => $trend, "reason" => $reason];
}

function traderKiana($data, $high, $low) {
    $keltnerLower = sma($data, 20) - 2 * sma($high, 14);
    $trend = (end($data) > $keltnerLower) ? "صعودی" : "نزولی";
    $reason = "قیمت " . ($trend === "صعودی" ? "بالای" : "پایین") . " کانال کلتنر است.";
    return ["name" => "کیانا", "trend" => $trend, "reason" => $reason];
}

function traderSina($data) {
    $trend = (end($data) > sma($data, 5)) ? "صعودی" : "نزولی";
    $reason = "قیمت نسبت به میانگین کوتاه‌مدت تغییر کرده است.";
    return ["name" => "سینا", "trend" => $trend, "reason" => $reason];
}

// --- اجرای تمام تریدرها ---
function runAllTraders($data, $high, $low) {
    return [
        traderAli($data, $high, $low),
        traderReza($data, $high, $low),
        traderSara($data),
        traderNeda($data, $high, $low),
        traderMohammad($data),
        traderHamed($data, $high, $low),
        traderFarhad($data),
        traderArash($data, $high, $low),
        traderElham($data),
        traderShirin($data, $high, $low),
        traderKiana($data, $high, $low),
        traderSina($data)
    ];
}

// --- اجرای فایل ---
if (isset($_GET['symbol']) && isset($_GET['interval'])) {
    $symbol = $_GET['symbol'];
    $interval = $_GET['interval'];

    try {
        $validIntervals = ['30m', '1h', '2h', '4h', '8h', '1d'];
        if (!in_array($interval, $validIntervals)) {
            throw new Exception("بازه زمانی نامعتبر است: $interval");
        }

        list($closePrices, $highPrices, $lowPrices, $timestamps) = fetchBinanceData($symbol, $interval, 100);

        $results = runAllTraders($closePrices, $highPrices, $lowPrices);

        foreach ($results as &$result) {
            $latestTimestamp = end($timestamps);
            $result['utcTime'] = date("Y-m-d H:i:s", $latestTimestamp);
            $result['tehranTime'] = date("Y-m-d H:i:s", $latestTimestamp + (3.5 * 3600));
        }

        header('Content-Type: application/json');
        echo json_encode($results, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
    } catch (Exception $e) {
        header('Content-Type: application/json', true, 500);
        echo json_encode(['error' => $e->getMessage()]);
    }
} else {
    header('Content-Type: application/json', true, 400);
    echo json_encode(['error' => 'پارامتر symbol یا interval ارسال نشده است.']);
}