<?php
// فعال‌سازی خطاها
ini_set('display_errors', 1);
error_reporting(E_ALL);

// وارد کردن فایل تریدرها
include 'traders.php';

// تابع برای دریافت داده‌ها از بایننس
function fetchPriceData($symbol, $interval) {
    $url = "https://api.binance.us/api/v3/klines?symbol=$symbol&interval=$interval";

    $response = @file_get_contents($url);
    if (!$response) {
        die("خطا در دریافت داده‌ها از Binance.US");
    }

    return json_decode($response, true);
}

// دریافت داده‌های بیت‌کوین در تایم‌فریم 15 دقیقه
$data = fetchPriceData("BTCUSDT", "15m");

// استخراج قیمت‌های بسته‌شدن، بالاترین و پایین‌ترین قیمت‌ها
$closingPrices = array_map(fn($candle) => floatval($candle[4]), $data);
$highPrices = array_map(fn($candle) => floatval($candle[2]), $data);
$lowPrices = array_map(fn($candle) => floatval($candle[3]), $data);

// استخراج زمان کندل‌ها
$signalTimes = array_map(fn($candle) => date("Y-m-d H:i:s", $candle[0] / 1000), $data);

// اجرای تحلیل 12 تریدر
$traderResults = runAllTraders($closingPrices, $highPrices, $lowPrices);

// آماده‌سازی پیام نهایی برای ارسال به وب‌هوک
$finalSignal = [
    'currency' => 'BTC',
    'time' => end($signalTimes),
    'results' => $traderResults
];

// ارسال سیگنال به وب‌هوک
$webhookUrl = "http://bot.mohtavanevisan.ir:8443/webhook";
$options = [
    'http' => [
        'header'  => "Content-type: application/json\r\n",
        'method'  => 'POST',
        'content' => json_encode($finalSignal),
    ],
];
$context = stream_context_create($options);
@file_get_contents($webhookUrl, false, $context);
?>

<!DOCTYPE html>
<html lang="fa">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>تحلیل بیت‌کوین</title>
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">
    <script src="https://cdn.jsdelivr.net/npm/chart.js"></script>
</head>
<body class="bg-light">
    <div class="container mt-5">
        <h1 class="text-center mb-4">نمودار و تحلیل بیت‌کوین</h1>

        <!-- نمودار -->
        <div class="card mb-4">
            <div class="card-header text-center">
                <strong>نمودار قیمت بیت‌کوین (15 دقیقه)</strong>
            </div>
            <div class="card-body">
                <canvas id="chart" width="800" height="400"></canvas>
            </div>
        </div>

        <!-- نتایج تحلیل -->
        <div class="card">
            <div class="card-header text-center">
                <strong>نتایج تحلیل 12 تریدر</strong>
            </div>
            <div class="card-body">
                <div class="row">
                    <?php foreach ($traderResults as $index => $result): ?>
                        <div class="col-md-6 mb-3">
                            <div class="card">
                                <div class="card-body">
                                    <h5 class="card-title">تریدر <?php echo $index + 1; ?></h5>
                                    <p class="card-text">
                                        <strong>وضعیت:</strong> <?php echo $result['trend']; ?><br>
                                        <strong>دلیل:</strong> <?php echo $result['reason']; ?><br>
                                        <strong>زمان سیگنال:</strong> <?php echo end($signalTimes); ?>
                                    </p>
                                </div>
                            </div>
                        </div>
                    <?php endforeach; ?>
                </div>
            </div>
        </div>
    </div>

    <script>
        const labels = <?php echo json_encode($signalTimes); ?>;
        const data = <?php echo json_encode($closingPrices); ?>;

        const chartData = {
            labels: labels,
            datasets: [{
                label: 'قیمت بیت‌کوین',
                data: data,
                borderColor: 'rgba(75, 192, 192, 1)',
                borderWidth: 2,
                fill: false
            }]
        };

        const config = {
            type: 'line',
            data: chartData,
            options: {
                responsive: true,
                plugins: {
                    legend: {
                        position: 'top',
                    },
                    title: {
                        display: true,
                        text: 'نمودار قیمت بیت‌کوین'
                    }
                },
                scales: {
                    x: {
                        display: true,
                        title: {
                            display: true,
                            text: 'زمان'
                        }
                    },
                    y: {
                        display: true,
                        title: {
                            display: true,
                            text: 'قیمت (USDT)'
                        }
                    }
                }
            }
        };

        new Chart(document.getElementById('chart'), config);
    </script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/js/bootstrap.bundle.min.js"></script>
</body>
</html>
