<?php
error_reporting(E_ALL);
ini_set('display_errors', 1);

// --- ابزارهای کمکی ---
function fetchBinanceData($symbol, $interval, $limit) {
    $calculatedLimit = intval($limit);

    if ($calculatedLimit <= 0 || $calculatedLimit > 1000) {
        throw new Exception("مقدار limit نامعتبر است.");
    }

    $url = "https://api.binance.com/api/v3/klines?symbol=$symbol&interval=$interval&limit=$calculatedLimit";

    $response = @file_get_contents($url);
    if ($response === false) {
        throw new Exception("خطا در اتصال به API بایننس.");
    }

    $data = json_decode($response, true);
    if (!is_array($data) || count($data) < $limit) {
        throw new Exception("خطا در دریافت داده‌ها از API بایننس.");
    }

    $closePrices = [];
    $highPrices = [];
    $lowPrices = [];
    $timestamps = [];

    foreach ($data as $item) {
        $timestamps[] = $item[0] / 1000; // تبدیل به ثانیه
        $closePrices[] = floatval($item[4]);
        $highPrices[] = floatval($item[2]);
        $lowPrices[] = floatval($item[3]);
    }

    if (count($closePrices) < $limit) {
        throw new Exception("تعداد داده‌ها کافی نیست.");
    }

    return [$closePrices, $highPrices, $lowPrices, $timestamps];
}

function ema($data, $period) {
    if (count($data) < $period) {
        throw new Exception("EMA: داده‌ها کافی نیستند یا تعداد داده‌ها کمتر از دوره مشخص شده است.");
    }

    $k = 2 / ($period + 1);
    $ema = $data[0];
    for ($i = 1; $i < count($data); $i++) {
        $ema = ($data[$i] * $k) + ($ema * (1 - $k));
    }
    return $ema;
}

function sma($data, $period) {
    if (count($data) < $period) {
        throw new Exception("SMA: داده‌ها کافی نیستند یا تعداد داده‌ها کمتر از دوره مشخص شده است.");
    }

    return array_sum(array_slice($data, -$period)) / $period;
}

function traderAliPlus($data, $high, $low) {
    $ema9 = ema($data, 9);
    $ema21 = ema($data, 21);
    $trend = ($ema9 > $ema21) ? "صعودی" : "نزولی";
    $reason = "EMA 9 " . ($trend === "صعودی" ? "بالای" : "پایین") . " EMA 21 است.";
    return ["name" => "علی (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderRezaPlus($data, $high, $low) {
    $pivotHigh = max($high);
    $pivotLow = min($low);
    $range = $pivotHigh - $pivotLow;
    $trend = (end($data) > $pivotHigh - 0.382 * $range) ? "صعودی" : "نزولی";
    $reason = "قیمت به سطح فیبوناچی نزدیک است.";
    return ["name" => "رضا (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderSaraPlus($data) {
    $rsi = sma($data, 14); // نمونه ساده RSI
    $trend = ($rsi > 70) ? "بیش خرید" : (($rsi < 30) ? "بیش فروش" : "خنثی");
    $reason = "RSI در محدوده " . $trend . " است.";
    return ["name" => "سارا (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderNedaPlus($data, $high, $low) {
    $atr = sma($high, 14) - sma($low, 14);
    $upperBand = sma($data, 20) + 2 * $atr;
    $lowerBand = sma($data, 20) - 2 * $atr;
    $trend = (end($data) > $upperBand) ? "صعودی" : ((end($data) < $lowerBand) ? "نزولی" : "خنثی");
    $reason = "قیمت در محدوده باندهای ATR است.";
    return ["name" => "ندا (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderMohammadPlus($data, $high, $low) {
    $vwap = sma($data, 20);
    $trend = (end($data) > $vwap) ? "صعودی" : "نزولی";
    $reason = "قیمت " . ($trend === "صعودی" ? "بالای" : "پایین") . " VWAP است.";
    return ["name" => "محمد (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderHamedPlus($data) {
    $macdLine = ema($data, 12) - ema($data, 26);
    $signalLine = ema(array_slice($data, -9), 9);
    $trend = ($macdLine > $signalLine) ? "صعودی" : "نزولی";
    $reason = "خط MACD " . ($trend === "صعودی" ? "بالای" : "پایین") . " خط سیگنال است.";
    return ["name" => "حامد (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderFarhadPlus($data, $high, $low) {
    $bollingerUpper = sma($data, 20) + 2 * sma($high, 20);
    $bollingerLower = sma($data, 20) - 2 * sma($low, 20);
    $trend = (end($data) > $bollingerUpper) ? "صعودی" : ((end($data) < $bollingerLower) ? "نزولی" : "خنثی");
    $reason = "قیمت در باند بولینگر است.";
    return ["name" => "فرهاد (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderArashPlus($data) {
    $roc = (end($data) - $data[count($data) - 10]) / $data[count($data) - 10];
    $trend = ($roc > 0) ? "صعودی" : "نزولی";
    $reason = "نرخ تغییر (ROC) " . ($trend === "صعودی" ? "مثبت" : "منفی") . " است.";
    return ["name" => "آرش (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderElhamPlus($data) {
    $adx = sma($data, 14); // نمونه ADX ساده
    $trend = ($adx > 25) ? "روند قوی" : "روند ضعیف";
    $reason = "ADX نشان‌دهنده " . $trend . " است.";
    return ["name" => "الهام (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderShirinPlus($data, $high, $low) {
    $ichimoku = sma($data, 9) > sma($high, 26); // نمونه ساده Ichimoku
    $trend = ($ichimoku) ? "صعودی" : "نزولی";
    $reason = "ابر ایچیموکو نشان‌دهنده " . $trend . " است.";
    return ["name" => "شیرین (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderKianaPlus($data, $high, $low) {
    $supertrend = sma($high, 10) - sma($low, 10);
    $trend = (end($data) > $supertrend) ? "صعودی" : "نزولی";
    $reason = "سیگنال سوپرترند صادر شده است.";
    return ["name" => "کیانا (Plus)", "trend" => $trend, "reason" => $reason];
}

function traderSinaPlus($data) {
    $keltnerUpper = sma($data, 20) + 2 * sma($data, 10);
    $trend = (end($data) > $keltnerUpper) ? "صعودی" : "نزولی";
    $reason = "قیمت در کانال کلتنر است.";
    return ["name" => "سینا (Plus)", "trend" => $trend, "reason" => $reason];
}

// --- اجرای تمام تریدرها ---
function runPlusTraders($data, $high, $low) {
    return [
        traderAliPlus($data, $high, $low),
        traderRezaPlus($data, $high, $low),
        traderSaraPlus($data),
        traderNedaPlus($data, $high, $low),
        traderMohammadPlus($data, $high, $low),
        traderHamedPlus($data),
        traderFarhadPlus($data, $high, $low),
        traderArashPlus($data),
        traderElhamPlus($data),
        traderShirinPlus($data, $high, $low),
        traderKianaPlus($data, $high, $low),
        traderSinaPlus($data),
    ];
}

// --- اجرای فایل ---
if (isset($_GET['symbol']) && isset($_GET['interval'])) {
    $symbol = $_GET['symbol'];
    $interval = $_GET['interval'];

    try {
        $validIntervals = ['30m', '1h', '2h', '4h', '8h', '1d'];
        if (!in_array($interval, $validIntervals)) {
            throw new Exception("بازه زمانی نامعتبر است: $interval");
        }

        list($closePrices, $highPrices, $lowPrices, $timestamps) = fetchBinanceData($symbol, $interval, 100);

        $results = runPlusTraders($closePrices, $highPrices, $lowPrices);

        foreach ($results as &$result) {
            $latestTimestamp = end($timestamps);
            $result['utcTime'] = date("Y-m-d H:i:s", $latestTimestamp);
            $result['tehranTime'] = date("Y-m-d H:i:s", $latestTimestamp + (3.5 * 3600));
        }

        header('Content-Type: application/json');
        echo json_encode($results, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
    } catch (Exception $e) {
        header('Content-Type: application/json', true, 500);
        echo json_encode(['error' => $e->getMessage()]);
    }
} else {
    header('Content-Type: application/json', true, 400);
    echo json_encode(['error' => 'پارامتر symbol یا interval ارسال نشده است.']);
}
?>
