<?php
error_reporting(E_ALL);
ini_set('display_errors', 1);

function fetchBinanceData($symbol, $interval, $limit) {
    $calculatedLimit = intval($limit);

    if ($calculatedLimit <= 0 || $calculatedLimit > 1000) {
        throw new Exception("مقدار limit نامعتبر است.");
    }

    $url = "https://api.binance.com/api/v3/klines?symbol=$symbol&interval=$interval&limit=$calculatedLimit";

    $response = @file_get_contents($url);
    if ($response === false) {
        throw new Exception("خطا در اتصال به API بایننس.");
    }

    $data = json_decode($response, true);
    if (!is_array($data) || count($data) < $limit) {
        throw new Exception("خطا در دریافت داده‌ها از API بایننس.");
    }

    $closePrices = [];
    $highPrices = [];
    $lowPrices = [];
    $timestamps = [];

    foreach ($data as $item) {
        $timestamps[] = $item[0] / 1000; // تبدیل به ثانیه
        $closePrices[] = floatval($item[4]);
        $highPrices[] = floatval($item[2]);
        $lowPrices[] = floatval($item[3]);
    }

    if (count($closePrices) < $limit) {
        throw new Exception("تعداد داده‌ها کافی نیست.");
    }

    return [$closePrices, $highPrices, $lowPrices, $timestamps];
}

function ema($data, $period) {
    if (count($data) < $period) {
        throw new Exception("EMA: داده‌ها کافی نیستند یا تعداد داده‌ها کمتر از دوره مشخص شده است.");
    }

    $k = 2 / ($period + 1);
    $ema = $data[0];
    for ($i = 1; $i < count($data); $i++) {
        $ema = ($data[$i] * $k) + ($ema * (1 - $k));
    }
    return $ema;
}

function sma($data, $period) {
    if (count($data) < $period) {
        throw new Exception("SMA: داده‌ها کافی نیستند یا تعداد داده‌ها کمتر از دوره مشخص شده است.");
    }

    return array_sum(array_slice($data, -$period)) / $period;
}

// --- الگوریتم‌های Pro Traders ---
function traderAliPro($data, $high, $low) {
    $ema9 = ema($data, 9);
    $ema21 = ema($data, 21);
    $macdLine = ema($data, 12) - ema($data, 26);
    $signalLine = ema(array_slice($data, -9), 9);
    $trend = ($ema9 > $ema21 && $macdLine > $signalLine) ? "صعودی" : "نزولی";
    $reason = "EMA 9 بالای EMA 21 و MACD بالای سیگنال است.";
    return ["name" => "علی (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderRezaPro($data, $high, $low) {
    $pivotHigh = max($high);
    $pivotLow = min($low);
    $fibonacciRetracement = $pivotHigh - 0.618 * ($pivotHigh - $pivotLow);
    $trend = (end($data) > $fibonacciRetracement) ? "صعودی" : "نزولی";
    $reason = "قیمت نزدیک به سطح فیبوناچی 61.8% است.";
    return ["name" => "رضا (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderSaraPro($data) {
    $rsi = sma($data, 14); // نمونه ساده RSI
    $adx = sma($data, 14); // نمونه ساده ADX
    $trend = ($rsi > 50 && $adx > 25) ? "صعودی" : "نزولی";
    $reason = "RSI بالای 50 و ADX نشان‌دهنده روند قوی است.";
    return ["name" => "سارا (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderNedaPro($data, $high, $low) {
    $upperBand = sma($data, 20) + 2 * (sma($high, 14) - sma($low, 14));
    $lowerBand = sma($data, 20) - 2 * (sma($high, 14) - sma($low, 14));
    $trend = (end($data) > $upperBand) ? "صعودی" : ((end($data) < $lowerBand) ? "نزولی" : "خنثی");
    $reason = "قیمت در محدوده باند بولینگر است.";
    return ["name" => "ندا (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderMohammadPro($data, $high, $low) {
    $vwap = sma($data, 20);
    $rsi = sma($data, 14); // نمونه ساده RSI
    $trend = (end($data) > $vwap && $rsi > 50) ? "صعودی" : "نزولی";
    $reason = "قیمت بالای VWAP و RSI بالای 50 است.";
    return ["name" => "محمد (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderHamedPro($data) {
    $macdLine = ema($data, 12) - ema($data, 26);
    $signalLine = ema(array_slice($data, -9), 9);
    $roc = (end($data) - $data[count($data) - 10]) / $data[count($data) - 10];
    $trend = ($macdLine > $signalLine && $roc > 0) ? "صعودی" : "نزولی";
    $reason = "MACD بالای سیگنال و نرخ تغییر (ROC) مثبت است.";
    return ["name" => "حامد (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderFarhadPro($data, $high, $low) {
    $atr = sma($high, 14) - sma($low, 14);
    $supertrend = sma($high, 10) - sma($low, 10);
    $trend = (end($data) > $supertrend && $atr < 5) ? "صعودی" : "نزولی";
    $reason = "سیگنال سوپرترند و ATR کمتر از 5 است.";
    return ["name" => "فرهاد (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderArashPro($data, $high, $low) {
    $bollingerUpper = sma($data, 20) + 2 * sma($high, 20);
    $bollingerLower = sma($data, 20) - 2 * sma($low, 20);
    $keltnerUpper = sma($data, 20) + 2 * sma($data, 10);
    $trend = (end($data) > $bollingerUpper && end($data) > $keltnerUpper) ? "صعودی" : "نزولی";
    $reason = "قیمت بالای باند بولینگر و کانال کلتنر است.";
    return ["name" => "آرش (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderElhamPro($data) {
    $adx = sma($data, 14); // نمونه ADX ساده
    $trend = ($adx > 25) ? "روند قوی" : "روند ضعیف";
    $reason = "ADX نشان‌دهنده " . $trend . " است.";
    return ["name" => "الهام (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderShirinPro($data, $high, $low) {
    $ichimoku = sma($data, 9) > sma($high, 26); // نمونه ساده Ichimoku
    $trend = ($ichimoku) ? "صعودی" : "نزولی";
    $reason = "ابر ایچیموکو نشان‌دهنده " . $trend . " است.";
    return ["name" => "شیرین (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderKianaPro($data, $high, $low) {
    $fibonacciSupport = min($low) + 0.382 * (max($high) - min($low));
    $trend = (end($data) > $fibonacciSupport) ? "صعودی" : "نزولی";
    $reason = "قیمت نزدیک سطح حمایت فیبوناچی است.";
    return ["name" => "کیانا (Pro)", "trend" => $trend, "reason" => $reason];
}

function traderSinaPro($data) {
    $roc = (end($data) - $data[count($data) - 14]) / $data[count($data) - 14];
    $adx = sma($data, 14); // نمونه ADX ساده
    $trend = ($roc > 0 && $adx > 25) ? "صعودی" : "نزولی";
    $reason = "ROC مثبت و ADX نشان‌دهنده روند قوی است.";
    return ["name" => "سینا (Pro)", "trend" => $trend, "reason" => $reason];
}

// --- اجرای تمام تریدرها ---
function runProTraders($data, $high, $low) {
    return [
        traderAliPro($data, $high, $low),
        traderRezaPro($data, $high, $low),
        traderSaraPro($data),
        traderNedaPro($data, $high, $low),
        traderMohammadPro($data, $high, $low),
        traderHamedPro($data),
        traderFarhadPro($data, $high, $low),
        traderArashPro($data, $high, $low),
        traderElhamPro($data),
        traderShirinPro($data, $high, $low),
        traderKianaPro($data, $high, $low),
        traderSinaPro($data),
    ];
}

// --- اجرای فایل ---
if (isset($_GET['symbol']) && isset($_GET['interval'])) {
    $symbol = $_GET['symbol'];
    $interval = $_GET['interval'];

    try {
        $validIntervals = ['30m', '1h', '2h', '4h', '8h', '1d'];
        if (!in_array($interval, $validIntervals)) {
            throw new Exception("بازه زمانی نامعتبر است: $interval");
        }

        list($closePrices, $highPrices, $lowPrices, $timestamps) = fetchBinanceData($symbol, $interval, 100);

        $results = runProTraders($closePrices, $highPrices, $lowPrices);

        foreach ($results as &$result) {
            $latestTimestamp = end($timestamps);
            $result['utcTime'] = date("Y-m-d H:i:s", $latestTimestamp);
            $result['tehranTime'] = date("Y-m-d H:i:s", $latestTimestamp + (3.5 * 3600));
        }

        header('Content-Type: application/json');
        echo json_encode($results, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
    } catch (Exception $e) {
        header('Content-Type: application/json', true, 500);
        echo json_encode(['error' => $e->getMessage()]);
    }
} else {
    header('Content-Type: application/json', true, 400);
    echo json_encode(['error' => 'پارامتر symbol یا interval ارسال نشده است.']);
}
?>